@@ -22,6 +22,7 @@
 #define __LIBTRACKER_DATA_UPDATE_H__
 
 #include <glib.h>
+#include <gio/gio.h>
 
 #include <libtracker-common/tracker-ontologies.h>
 
@@ -33,16 +34,6 @@ G_BEGIN_DECLS
 #error "only <libtracker-data/tracker-data.h> must be included directly."
 #endif
 
-#define TRACKER_DATA_ERROR tracker_data_error_quark ()
-
-typedef enum  {
-	TRACKER_DATA_ERROR_UNKNOWN_CLASS,
-	TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
-	TRACKER_DATA_ERROR_INVALID_TYPE,
-	TRACKER_DATA_ERROR_CONSTRAINT,
-	TRACKER_DATA_ERROR_NO_SPACE
-} TrackerDataError;
-
 typedef void (*TrackerStatementCallback) (const gchar *graph,
                                           const gchar *subject,
                                           const gchar *predicate,
@@ -80,12 +71,11 @@ void     tracker_data_insert_statement_with_string  (const gchar               *
                                                      const gchar               *predicate,
                                                      const gchar               *object,
                                                      GError                   **error);
-void     tracker_data_begin_db_transaction          (void);
-void     tracker_data_begin_db_transaction_for_replay (time_t                   time);
-void     tracker_data_commit_db_transaction         (void);
-void     tracker_data_notify_db_transaction         (void);
 void     tracker_data_begin_transaction             (GError                   **error);
+void     tracker_data_begin_transaction_for_replay  (time_t                     time,
+                                                     GError                   **error);
 void     tracker_data_commit_transaction            (GError                   **error);
+void     tracker_data_notify_transaction            (void);
 void     tracker_data_rollback_transaction          (void);
 void     tracker_data_update_sparql                 (const gchar               *update,
                                                      GError                   **error);
@@ -94,6 +84,8 @@ GPtrArray *
                                                      GError                   **error);
 void     tracker_data_update_buffer_flush           (GError                   **error);
 void     tracker_data_update_buffer_might_flush     (GError                   **error);
+void     tracker_data_load_turtle_file              (GFile                     *file,
+                                                     GError                   **error);
 
 void     tracker_data_sync                          (void);
 void     tracker_data_replay_journal                (GHashTable                *classes,
