@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,12 @@
 #include "tracker-data-manager.h"
 #include "tracker-data-query.h"
 #include "tracker-data-update.h"
+#include "tracker-db-config.h"
+#include "tracker-db-dbus.h"
+#include "tracker-db-interface.h"
+#include "tracker-db-interface-sqlite.h"
+#include "tracker-db-journal.h"
+#include "tracker-db-manager.h"
 #include "tracker-namespace.h"
 #include "tracker-ontology.h"
 #include "tracker-ontologies.h"
