@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,8 +35,16 @@ G_BEGIN_DECLS
 #define TRACKER_IS_MINER_MANAGER_CLASS(c)  (G_TYPE_CHECK_CLASS_TYPE ((c),    TRACKER_TYPE_MINER_MANAGER))
 #define TRACKER_MINER_MANAGER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o),  TRACKER_TYPE_MINER_MANAGER, TrackerMinerManagerClass))
 
+#define TRACKER_MINER_MANAGER_ERROR tracker_miner_manager_error_quark ()
+
 typedef struct TrackerMinerManager TrackerMinerManager;
 typedef struct TrackerMinerManagerClass TrackerMinerManagerClass;
+typedef enum TrackerMinerManagerError TrackerMinerManagerError;
+
+enum TrackerMinerManagerError {
+	TRACKER_MINER_MANAGER_ERROR_NOT_AVAILABLE,
+	TRACKER_MINER_MANAGER_ERROR_NOENT
+};
 
 /**
  * TrackerMinerManager:
@@ -70,6 +78,7 @@ struct TrackerMinerManagerClass {
 };
 
 GType                tracker_miner_manager_get_type           (void) G_GNUC_CONST;
+GQuark               tracker_miner_manager_error_quark        (void) G_GNUC_CONST;
 
 TrackerMinerManager *tracker_miner_manager_new                (void);
 GSList *             tracker_miner_manager_get_running        (TrackerMinerManager  *manager);
@@ -99,6 +108,13 @@ const gchar *        tracker_miner_manager_get_display_name   (TrackerMinerManag
 const gchar *        tracker_miner_manager_get_description    (TrackerMinerManager  *manager,
                                                                const gchar          *miner);
 
+gboolean             tracker_miner_manager_reindex_by_mimetype (TrackerMinerManager  *manager,
+                                                                const GStrv           mimetypes,
+                                                                GError              **error);
+gboolean             tracker_miner_manager_index_file          (TrackerMinerManager  *manager,
+                                                                GFile                *file,
+                                                                GError              **error);
+
 G_END_DECLS
 
 #endif /* __LIBTRACKER_MINER_MANAGER_H__ */
