@@ -38,7 +38,7 @@
 #include <libtracker-data/tracker-db-manager.h>
 #include <libtracker-miner/tracker-miner.h>
 
-#include "tracker-miner-files-reindex-client.h"
+#include "tracker-miner-files-index-client.h"
 
 #define ABOUT	  \
 	"Tracker " PACKAGE_VERSION "\n"
@@ -69,6 +69,7 @@ static gboolean remove_config;
 static gboolean start;
 static const gchar **reindex_mime_types;
 static gboolean print_version;
+static gchar *index_file;
 
 static gboolean term_option_arg_func (const gchar  *option_value,
                                       const gchar  *value,
@@ -97,6 +98,9 @@ static GOptionEntry entries[] = {
 	{ "reindex-mime-type", 'm', 0, G_OPTION_ARG_STRING_ARRAY, &reindex_mime_types,
 	  N_("Reindex files which match the mime type supplied (for new extractors), use -m MIME1 -m MIME2"),
 	  N_("MIME") },
+	{ "index-file", 'f', 0, G_OPTION_ARG_FILENAME, &index_file,
+	  N_("(Re)Index a given file"),
+	  N_("FILE") },
 	{ "version", 'V', 0, G_OPTION_ARG_NONE, &print_version,
 	  N_("Print version"),
 	  NULL },
@@ -299,7 +303,8 @@ main (int argc, char **argv)
 	 */
 	if (kill_option != TERM_NONE ||
 	    terminate_option != TERM_NONE ||
-	    (!start && !remove_config)) {
+	    (!start && !remove_config && !reindex_mime_types && 
+	     !print_version && !index_file)) {
 		pids = get_pids ();
 		str = g_strdup_printf (g_dngettext (NULL,
 		                                    "Found %d PID…",
@@ -532,49 +537,50 @@ main (int argc, char **argv)
 		}
 
 		g_slist_free (miners);
+		g_object_unref (manager);
 	}
 
 	if (reindex_mime_types) {
-		DBusGConnection *connection;
-		DBusGProxy *proxy;
-	
+		TrackerMinerManager *manager;
+
+		manager = tracker_miner_manager_new ();
+		tracker_miner_manager_reindex_by_mimetype (manager, reindex_mime_types, &error);
+
+		if (error) {
+			g_printerr ("%s: %s\n",
+			            _("Could not reindex mimetypes"),
+			            error->message);
+			g_error_free (error);
+			return EXIT_FAILURE;
+		}
+
+		g_print ("%s\n", _("Reindexing mime types was successful"));
+
+		g_object_unref (manager);
+	}
+
+	if (index_file) {
+		TrackerMinerManager *manager;
 		GError *error = NULL;
+		GFile *file;
 
-		connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-		
-		if (!connection) {
-			g_print ("Could not connect to the D-Bus session bus, %s",
-			         error ? error->message : "no error given.");
-			g_clear_error (&error);
-			return FALSE;
-		}
-		
-		/* The definitions below (DBUS_SERVICE_DBUS, etc) are
-		 * predefined for us to just use (dbus_g_proxy_...)
-		 */
-		proxy = dbus_g_proxy_new_for_name (connection,
-		                                   "org.freedesktop.Tracker1.Miner.Files.Reindex",
-		                                   "/org/freedesktop/Tracker1/Miner/Files/Reindex",
-		                                   "org.freedesktop.Tracker1.Miner.Files.Reindex");
-		
-		if (!proxy) {
-			g_print ("Could not create a proxy for the D-Bus service, %s",
-			         error ? error->message : "no error given.");
-			g_clear_error (&error);
-			return FALSE;
-		}
-		
-		if (!org_freedesktop_Tracker1_Miner_Files_Reindex_mime_types (proxy, 
-		                                                              reindex_mime_types, 
-		                                                              &error)) {
-			g_print ("Could not reindex mime types, %s",
-			         error ? error->message : "no error given.");
-			g_clear_error (&error);
-			g_object_unref (proxy);
-			return FALSE;
+		file = g_file_new_for_commandline_arg (index_file);
+		manager = tracker_miner_manager_new ();
+
+		tracker_miner_manager_index_file (manager, file, &error);
+
+		if (error) {
+			g_printerr ("%s: %s\n",
+			            _("Could not (re)index file"),
+			            error->message);
+			g_error_free (error);
+			return EXIT_FAILURE;
 		}
 
-		g_object_unref (proxy);
+		g_print ("%s\n", _("(Re)indexing file was successful"));
+
+		g_object_unref (manager);
+		g_object_unref (file);
 	}
 
 	return EXIT_SUCCESS;
