@@ -45,8 +45,6 @@
 
 #include <libtracker-common/tracker-common.h>
 
-#include <libtracker-client/tracker.h>
-
 #include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-albumart.h"
@@ -87,6 +85,7 @@ typedef struct {
 	gchar *comment;
 	gchar *content_type;
 	gchar *copyright;
+	gchar *encoded_by;
 	guint32 length;
 	gchar *performer1;
 	gchar *performer2;
@@ -109,6 +108,7 @@ typedef enum {
 	ID3V2_TAL,
 	ID3V2_TCO,
 	ID3V2_TCR,
+	ID3V2_TEN,
 	ID3V2_TLE,
 	ID3V2_TPB,
 	ID3V2_TP1,
@@ -125,22 +125,23 @@ typedef enum {
 	ID3V24_APIC,
 	ID3V24_COMM,
 	ID3V24_TALB,
+	ID3V24_TCOM,
 	ID3V24_TCON,
 	ID3V24_TCOP,
 	ID3V24_TDRC,
 	ID3V24_TDRL,
+	ID3V24_TENC,
 	ID3V24_TEXT,
 	ID3V24_TIT1,
 	ID3V24_TIT2,
 	ID3V24_TIT3,
 	ID3V24_TLEN,
+	ID3V24_TOLY,
 	ID3V24_TPE1,
 	ID3V24_TPE2,
 	ID3V24_TPUB,
 	ID3V24_TRCK,
 	ID3V24_TYER,
-	ID3V24_TOLY,
-	ID3V24_TCOM,
 } id3v24frame;
 
 typedef struct {
@@ -149,20 +150,21 @@ typedef struct {
 
 	guint32 duration;
 
-	gchar *title;
-	gchar *performer;
+	const gchar *title;
+	const gchar *performer;
 	gchar *performer_uri;
-	gchar *lyricist;
+	const gchar *lyricist;
 	gchar *lyricist_uri;
-	gchar *album;
+	const gchar *album;
 	gchar *album_uri;
-	gchar *genre;
-	gchar *text;
-	gchar *recording_time;
-	gchar *copyright;
-	gchar *publisher;
-	gchar *comment;
-	gchar *composer;
+	const gchar *genre;
+	const gchar *text;
+	const gchar *recording_time;
+	const gchar *encoded_by;
+	const gchar *copyright;
+	const gchar *publisher;
+	const gchar *comment;
+	const gchar *composer;
 	gchar *composer_uri;
 	gint track_number;
 	gint track_count;
@@ -208,12 +210,13 @@ static const struct {
 	{ "TCOP", ID3V24_TCOP },
 	{ "TDRC", ID3V24_TDRC },
 	{ "TDRL", ID3V24_TDRL },
-	{ "TOLY", ID3V24_TOLY },
+	{ "TENC", ID3V24_TENC },
 	{ "TEXT", ID3V24_TEXT },
 	{ "TIT1", ID3V24_TIT1 },
 	{ "TIT2", ID3V24_TIT2 },
 	{ "TIT3", ID3V24_TIT3 },
 	{ "TLEN", ID3V24_TLEN },
+	{ "TOLY", ID3V24_TOLY },
 	{ "TPE1", ID3V24_TPE1 },
 	{ "TPE2", ID3V24_TPE2 },
 	{ "TPUB", ID3V24_TPUB },
@@ -231,6 +234,7 @@ static const struct {
 	{ "TAL", ID3V2_TAL },
 	{ "TCO", ID3V2_TCO },
 	{ "TCR", ID3V2_TCR },
+	{ "TEN", ID3V2_TEN },
 	{ "TLE", ID3V2_TLE },
 	{ "TP1", ID3V2_TP1 },
 	{ "TP2", ID3V2_TP2 },
@@ -436,6 +440,41 @@ static TrackerExtractData extract_data[] = {
 	{ NULL, NULL }
 };
 
+
+
+static void
+id3tag_free (id3tag *tags)
+{
+	g_free (tags->title);
+	g_free (tags->artist);
+	g_free (tags->album);
+	g_free (tags->recording_time);
+	g_free (tags->comment);
+	g_free (tags->genre);
+	g_free (tags->encoding);
+}
+
+static void
+id3v2tag_free (id3v2tag *tags)
+{
+	g_free (tags->album);
+	g_free (tags->comment);
+	g_free (tags->content_type);
+	g_free (tags->copyright);
+	g_free (tags->performer1);
+	g_free (tags->performer2);
+	g_free (tags->composer);
+	g_free (tags->publisher);
+	g_free (tags->recording_time);
+	g_free (tags->release_time);
+	g_free (tags->encoded_by);
+	g_free (tags->text);
+	g_free (tags->toly);
+	g_free (tags->title1);
+	g_free (tags->title2);
+	g_free (tags->title3);
+}
+
 static char *
 read_id3v1_buffer (int     fd,
                    goffset size)
@@ -743,7 +782,7 @@ get_id3 (const gchar *data,
 
 	pos += 30;
 	year = g_convert (pos, 4, "UTF-8", encoding, NULL, NULL, NULL);
-	if (atoi (year) > 0) {
+	if (year && atoi (year) > 0) {
 		id3->recording_time = tracker_date_guess (year);
 	}
 	g_free (year);
@@ -903,8 +942,9 @@ mp3_parse_header (const gchar          *data,
 
 	if (filedata->duration == 0) {
 		if ((!vbr_flag && frames > VBR_THRESHOLD) || (frames > MAX_FRAMES_SCAN)) {
-			/* If not all frames scanned */
-			length = (filedata->size - filedata->id3v2_size) / (avg_bps ? avg_bps : bitrate ? bitrate : 0xFFFFFFFF) / 125;
+			/* If not all frames scanned
+			 * Note that bitrate is always > 0, checked before */
+			length = (filedata->size - filedata->id3v2_size) / (avg_bps ? avg_bps : bitrate) / 125;
 		} else{
 			length = spfp8 * 8 * frames / (sample_rate ? sample_rate : 0xFFFFFFFF);
 		}
@@ -1097,8 +1137,9 @@ id3v2_get_frame (const gchar *name)
 }
 
 static void
-get_id3v24_tags (const gchar          *data,
-                 size_t                size,
+get_id3v24_tags (id3v24frame           frame,
+                 const gchar          *data,
+                 size_t                csize,
                  id3tag               *info,
                  const gchar          *uri,
                  TrackerSparqlBuilder *metadata,
@@ -1107,212 +1148,168 @@ get_id3v24_tags (const gchar          *data,
 	id3v2tag *tag = &filedata->id3v24;
 	guint pos = 0;
 
-	while (pos < size) {
-		id3v24frame frame;
-		size_t csize;
-		unsigned short flags;
+	switch (frame) {
+	case ID3V24_APIC: {
+		/* embedded image */
+		gchar text_type;
+		const gchar *mime;
+		gchar pic_type;
+		const gchar *desc;
+		guint offset;
+		gint mime_len;
 
-		if (pos + 10 > size) {
-			return;
+		text_type =  data[pos + 0];
+		mime      = &data[pos + 1];
+		mime_len  = strlen (mime);
+		pic_type  =  data[pos + 1 + mime_len + 1];
+		desc      = &data[pos + 1 + mime_len + 1 + 1];
+
+		if (pic_type == 3 || (pic_type == 0 && filedata->albumart_size == 0)) {
+			offset = pos + 1 + mime_len + 2 + strlen (desc) + 1;
+
+			filedata->albumart_data = g_malloc0 (csize);
+			filedata->albumart_mime = g_strdup (mime);
+			memcpy (filedata->albumart_data, &data[offset], csize);
+			filedata->albumart_size = csize;
+		}
+		break;
+	}
+
+	case ID3V24_COMM: {
+		gchar *word;
+		gchar text_encode;
+		const gchar *text_language;
+		const gchar *text_desc;
+		const gchar *text;
+		guint offset;
+		gint text_desc_len;
+
+		text_encode   =  data[pos + 0]; /* $xx */
+		text_language = &data[pos + 1]; /* $xx xx xx */
+		text_desc     = &data[pos + 4]; /* <text string according to encoding> $00 (00) */
+		text_desc_len = strlen (text_desc);
+		text          = &data[pos + 4 + text_desc_len + 1]; /* <full text string according to encoding> */
+
+		offset = 4 + text_desc_len + 1;
+
+		word = id3v24_text_to_utf8 (text_encode, text, csize - offset);
+
+		if (!tracker_is_empty_string (word)) {
+			g_strstrip (word);
+			g_free (tag->comment);
+			tag->comment = word;
+		} else {
+			g_free (word);
+		}
+		break;
+	}
+
+	default: {
+		gchar *word;
+
+		/* text frames */
+		word = id3v24_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+		if (!tracker_is_empty_string (word)) {
+			g_strstrip (word);
 		}
 
-		frame = id3v24_get_frame (&data[pos]);
-
-		csize = (((data[pos+4] & 0x7F) << 21) |
-		         ((data[pos+5] & 0x7F) << 14) |
-		         ((data[pos+6] & 0x7F) << 7) |
-		         ((data[pos+7] & 0x7F) << 0));
-
-		flags = (((unsigned char) (data[pos + 8]) << 8) +
-		         ((unsigned char) (data[pos + 9])));
-
-		pos += 10;
-
-		if (frame == ID3V24_UNKNOWN) {
-			/* ignore unknown frames */
-			pos += csize;
-			continue;
-		}
-
-		if (pos + csize > size) {
-			break;
-		} else if (csize == 0) {
-			continue;
-		}
-
-		if (((flags & 0x80) > 0) ||
-		    ((flags & 0x40) > 0)) {
-			pos += csize;
-			continue;
-		}
-
-		if ((flags & 0x20) > 0) {
-			/* The "group" identifier, skip a byte */
-			pos++;
-			csize--;
-		}
+		g_debug ("Frame is %d, word is %s", frame, word);
 
 		switch (frame) {
-		case ID3V24_APIC: {
-			/* embedded image */
-			gchar text_type;
-			const gchar *mime;
-			gchar pic_type;
-			const gchar *desc;
-			guint offset;
-			gint mime_len;
-
-			text_type =  data[pos + 0];
-			mime      = &data[pos + 1];
-			mime_len  = strlen (mime);
-			pic_type  =  data[pos + 1 + mime_len + 1];
-			desc      = &data[pos + 1 + mime_len + 1 + 1];
-
-			if (pic_type == 3 || (pic_type == 0 && filedata->albumart_size == 0)) {
-				offset = pos + 1 + mime_len + 2 + strlen (desc) + 1;
-
-				filedata->albumart_data = g_malloc0 (csize);
-				filedata->albumart_mime = g_strdup (mime);
-				memcpy (filedata->albumart_data, &data[offset], csize);
-				filedata->albumart_size = csize;
-			}
+		case ID3V24_TALB:
+			tag->album = word;
 			break;
-		}
+		case ID3V24_TCON: {
+			gint genre;
 
-		case ID3V24_COMM: {
-			gchar *word;
-			gchar text_encode;
-			const gchar *text_language;
-			const gchar *text_desc;
-			const gchar *text;
-			guint offset;
-			gint text_desc_len;
-
-			text_encode   =  data[pos + 0]; /* $xx */
-			text_language = &data[pos + 1]; /* $xx xx xx */
-			text_desc     = &data[pos + 4]; /* <text string according to encoding> $00 (00) */
-			text_desc_len = strlen (text_desc);
-			text          = &data[pos + 4 + text_desc_len + 1]; /* <full text string according to encoding> */
-
-			offset = 4 + text_desc_len + 1;
-
-			word = id3v24_text_to_utf8 (text_encode, text, csize - offset);
-
-			if (!tracker_is_empty_string (word)) {
-				g_strstrip (word);
-				g_free (tag->comment);
-				tag->comment = word;
+			if (get_genre_number (word, &genre)) {
+				g_free (word);
+				word = g_strdup (get_genre_name (genre));
+			}
+			if (word && strcasecmp (word, "unknown") != 0) {
+				tag->content_type = word;
 			} else {
 				g_free (word);
 			}
 			break;
 		}
+		case ID3V24_TCOP:
+			tag->copyright = word;
+			break;
+		case ID3V24_TDRC:
+			tag->recording_time = tracker_date_guess (word);
+			g_free (word);
+			break;
+		case ID3V24_TDRL:
+			tag->release_time = tracker_date_guess (word);
+			g_free (word);
+			break;
+		case ID3V24_TENC:
+			tag->encoded_by = word;
+			break;
+		case ID3V24_TEXT:
+			tag->text = word;
+			break;
+		case ID3V24_TOLY:
+			tag->toly = word;
+			break;
+		case ID3V24_TCOM:
+			tag->composer = word;
+			break;
+		case ID3V24_TIT1:
+			tag->title1 = word;
+			break;
+		case ID3V24_TIT2:
+			tag->title2 = word;
+			break;
+		case ID3V24_TIT3:
+			tag->title3 = word;
+			break;
+		case ID3V24_TLEN:
+			tag->length = atoi (word) / 1000;
+			g_free (word);
+			break;
+		case ID3V24_TPE1:
+			tag->performer1 = word;
+			break;
+		case ID3V24_TPE2:
+			tag->performer2 = word;
+			break;
+		case ID3V24_TPUB:
+			tag->publisher = word;
+			break;
+		case ID3V24_TRCK: {
+			gchar **parts;
 
-		default: {
-			gchar *word;
-
-			/* text frames */
-			word = id3v24_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
-			if (!tracker_is_empty_string (word)) {
-				g_strstrip (word);
-			}
-
-			g_debug ("Frame is %d, word is %s", frame, word);
-
-			switch (frame) {
-			case ID3V24_TALB:
-				tag->album = word;
-				break;
-			case ID3V24_TCON: {
-				gint genre;
-
-				if (get_genre_number (word, &genre)) {
-					g_free (word);
-					word = g_strdup (get_genre_name (genre));
+			parts = g_strsplit (word, "/", 2);
+			if (parts[0]) {
+				tag->track_number = atoi (parts[0]);
+				if (parts[1]) {
+					tag->track_count = atoi (parts[1]);
 				}
-				if (word && strcasecmp (word, "unknown") != 0) {
-					tag->content_type = word;
-				} else {
-					g_free (word);
-				}
-				break;
 			}
-			case ID3V24_TCOP:
-				tag->copyright = word;
-				break;
-			case ID3V24_TDRC:
+			g_strfreev (parts);
+			g_free (word);
+
+			break;
+		}
+		case ID3V24_TYER:
+			if (atoi (word) > 0) {
 				tag->recording_time = tracker_date_guess (word);
-				g_free (word);
-				break;
-			case ID3V24_TDRL:
-				tag->release_time = tracker_date_guess (word);
-				g_free (word);
-				break;
-			case ID3V24_TEXT:
-				tag->text = word;
-				break;
-			case ID3V24_TOLY:
-				tag->toly = word;
-				break;
-			case ID3V24_TCOM:
-				tag->composer = word;
-				break;
-			case ID3V24_TIT1:
-				tag->title1 = word;
-				break;
-			case ID3V24_TIT2:
-				tag->title2 = word;
-				break;
-			case ID3V24_TIT3:
-				tag->title3 = word;
-				break;
-			case ID3V24_TLEN:
-				tag->length = atoi (word) / 1000;
-				g_free (word);
-				break;
-			case ID3V24_TPE1:
-				tag->performer1 = word;
-				break;
-			case ID3V24_TPE2:
-				tag->performer2 = word;
-				break;
-			case ID3V24_TPUB:
-				tag->publisher = word;
-				break;
-			case ID3V24_TRCK: {
-				gchar **parts;
-
-				parts = g_strsplit (word, "/", 2);
-				if (parts[0]) {
-					tag->track_number = atoi (parts[0]);
-					if (parts[1]) {
-						tag->track_count = atoi (parts[1]);
-					}
-				}
-				g_strfreev (parts);
-				g_free (word);
-
-				break;
-			}
-			case ID3V24_TYER:
-				if (atoi (word) > 0) {
-					tag->recording_time = tracker_date_guess (word);
-				}
-				g_free (word);
-				break;
-			default:
-				g_free (word);
 			}
+			g_free (word);
+			break;
+		default:
+			g_free (word);
 		}
-		}
-
-		pos += csize;
+	}
 	}
 }
 
 static void
-get_id3v23_tags (const gchar          *data,
-                 size_t                size,
+get_id3v23_tags (id3v24frame           frame,
+                 const gchar          *data,
+                 size_t                csize,
                  id3tag               *info,
                  const gchar          *uri,
                  TrackerSparqlBuilder *metadata,
@@ -1321,200 +1318,157 @@ get_id3v23_tags (const gchar          *data,
 	id3v2tag *tag = &filedata->id3v23;
 	guint pos = 0;
 
-	while (pos < size) {
-		id3v24frame frame;
-		size_t csize;
-		unsigned short flags;
+	switch (frame) {
+	case ID3V24_APIC: {
+		/* embedded image */
+		gchar text_type;
+		const gchar *mime;
+		gchar pic_type;
+		const gchar *desc;
+		guint offset;
+		gint  mime_len;
 
-		if (pos + 10 > size) {
-			return;
+		text_type =  data[pos + 0];
+		mime      = &data[pos + 1];
+		mime_len  = strlen (mime);
+		pic_type  =  data[pos + 1 + mime_len + 1];
+		desc      = &data[pos + 1 + mime_len + 1 + 1];
+
+		if (pic_type == 3 || (pic_type == 0 && filedata->albumart_size == 0)) {
+			offset = pos + 1 + mime_len + 2 + strlen (desc) + 1;
+
+			filedata->albumart_data = g_malloc0 (csize);
+			filedata->albumart_mime = g_strdup (mime);
+			memcpy (filedata->albumart_data, &data[offset], csize);
+			filedata->albumart_size = csize;
+		}
+		break;
+	}
+
+	case ID3V24_COMM: {
+		gchar *word;
+		gchar text_encode;
+		const gchar *text_language;
+		const gchar *text_desc;
+		const gchar *text;
+		guint offset;
+		gint text_desc_len;
+
+		text_encode   =  data[pos + 0]; /* $xx */
+		text_language = &data[pos + 1]; /* $xx xx xx */
+		text_desc     = &data[pos + 4]; /* <text string according to encoding> $00 (00) */
+		text_desc_len = strlen (text_desc);
+		text          = &data[pos + 4 + text_desc_len + 1]; /* <full text string according to encoding> */
+
+		offset = 4 + text_desc_len + 1;
+
+		word = id3v2_text_to_utf8 (text_encode, text, csize - offset);
+
+		if (!tracker_is_empty_string (word)) {
+			g_strstrip (word);
+			g_free (tag->comment);
+			tag->comment = word;
+		} else {
+			g_free (word);
 		}
 
-		frame = id3v24_get_frame (&data[pos]);
+		break;
+	}
 
-		csize = (((unsigned char)(data[pos + 4]) << 24) |
-		         ((unsigned char)(data[pos + 5]) << 16) |
-		         ((unsigned char)(data[pos + 6]) << 8)  |
-		         ((unsigned char)(data[pos + 7]) << 0) );
+	default: {
+		gchar *word;
 
-		flags = (((unsigned char)(data[pos + 8]) << 8) +
-		         ((unsigned char)(data[pos + 9])));
+		/* text frames */
+		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
 
-		pos += 10;
-
-		if (frame == ID3V24_UNKNOWN) {
-			/* ignore unknown frames */
-			pos += csize;
-			continue;
-		}
-
-		if (pos + csize > size) {
-			break;
-		} else if (csize == 0) {
-			continue;
-		}
-
-		if (((flags & 0x80) > 0) || ((flags & 0x40) > 0)) {
-			pos += csize;
-			continue;
-		}
-
-		if ((flags & 0x20) > 0) {
-			/* The "group" identifier, skip a byte */
-			pos++;
-			csize--;
+		if (!tracker_is_empty_string (word)) {
+			g_strstrip (word);
 		}
 
 		switch (frame) {
-		case ID3V24_APIC: {
-			/* embedded image */
-			gchar text_type;
-			const gchar *mime;
-			gchar pic_type;
-			const gchar *desc;
-			guint offset;
-			gint  mime_len;
-
-			text_type =  data[pos + 0];
-			mime      = &data[pos + 1];
-			mime_len  = strlen (mime);
-			pic_type  =  data[pos + 1 + mime_len + 1];
-			desc      = &data[pos + 1 + mime_len + 1 + 1];
-
-			if (pic_type == 3 || (pic_type == 0 && filedata->albumart_size == 0)) {
-				offset = pos + 1 + mime_len + 2 + strlen (desc) + 1;
-
-				filedata->albumart_data = g_malloc0 (csize);
-				filedata->albumart_mime = g_strdup (mime);
-				memcpy (filedata->albumart_data, &data[offset], csize);
-				filedata->albumart_size = csize;
-			}
+		case ID3V24_TALB:
+			tag->album = word;
 			break;
-		}
+		case ID3V24_TCON: {
+			gint genre;
 
-		case ID3V24_COMM: {
-			gchar *word;
-			gchar text_encode;
-			const gchar *text_language;
-			const gchar *text_desc;
-			const gchar *text;
-			guint offset;
-			gint text_desc_len;
-
-			text_encode   =  data[pos + 0]; /* $xx */
-			text_language = &data[pos + 1]; /* $xx xx xx */
-			text_desc     = &data[pos + 4]; /* <text string according to encoding> $00 (00) */
-			text_desc_len = strlen (text_desc);
-			text          = &data[pos + 4 + text_desc_len + 1]; /* <full text string according to encoding> */
-
-			offset = 4 + text_desc_len + 1;
-
-			word = id3v2_text_to_utf8 (text_encode, text, csize - offset);
-
-			if (!tracker_is_empty_string (word)) {
-				g_strstrip (word);
-				g_free (tag->comment);
-				tag->comment = word;
+			if (get_genre_number (word, &genre)) {
+				g_free (word);
+				word = g_strdup (get_genre_name (genre));
+			}
+			if (word && strcasecmp (word, "unknown") != 0) {
+				tag->content_type = word;
 			} else {
 				g_free (word);
 			}
+			break;
+		}
+		case ID3V24_TCOP:
+			tag->copyright = word;
+			break;
+		case ID3V24_TENC:
+			tag->encoded_by = word;
+			break;
+		case ID3V24_TEXT:
+			tag->text = word;
+			break;
+		case ID3V24_TOLY:
+			tag->toly = word;
+			break;
+		case ID3V24_TIT1:
+			tag->title1 = word;
+			break;
+		case ID3V24_TIT2:
+			tag->title2 = word;
+			break;
+		case ID3V24_TIT3:
+			tag->title3 = word;
+			break;
+		case ID3V24_TLEN:
+			tag->length = atoi (word) / 1000;
+			g_free (word);
+			break;
+		case ID3V24_TPE1:
+			tag->performer1 = word;
+			break;
+		case ID3V24_TPE2:
+			tag->performer2 = word;
+			break;
+		case ID3V24_TPUB:
+			tag->publisher = word;
+			break;
+		case ID3V24_TRCK: {
+			gchar **parts;
+
+			parts = g_strsplit (word, "/", 2);
+			if (parts[0]) {
+				tag->track_number = atoi (parts[0]);
+				if (parts[1]) {
+					tag->track_count = atoi (parts[1]);
+				}
+			}
+			g_strfreev (parts);
+			g_free (word);
 
 			break;
 		}
-
-		default: {
-			gchar *word;
-
-			/* text frames */
-			word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
-
-			if (!tracker_is_empty_string (word)) {
-				g_strstrip (word);
-			}
-
-			switch (frame) {
-			case ID3V24_TALB:
-				tag->album = word;
-				break;
-			case ID3V24_TCON: {
-				gint genre;
-
-				if (get_genre_number (word, &genre)) {
-					g_free (word);
-					word = g_strdup (get_genre_name (genre));
-				}
-				if (word && strcasecmp (word, "unknown") != 0) {
-					tag->content_type = word;
-				} else {
-					g_free (word);
-				}
-				break;
-			}
-			case ID3V24_TCOP:
-				tag->copyright = word;
-				break;
-			case ID3V24_TEXT:
-				tag->text = word;
-				break;
-			case ID3V24_TOLY:
-				tag->toly = word;
-				break;
-			case ID3V24_TIT1:
-				tag->title1 = word;
-				break;
-			case ID3V24_TIT2:
-				tag->title2 = word;
-				break;
-			case ID3V24_TIT3:
-				tag->title3 = word;
-				break;
-			case ID3V24_TLEN:
-				tag->length = atoi (word) / 1000;
-				g_free (word);
-				break;
-			case ID3V24_TPE1:
-				tag->performer1 = word;
-				break;
-			case ID3V24_TPE2:
-				tag->performer2 = word;
-				break;
-			case ID3V24_TPUB:
-				tag->publisher = word;
-				break;
-			case ID3V24_TRCK: {
-				gchar **parts;
-
-				parts = g_strsplit (word, "/", 2);
-				if (parts[0]) {
-					tag->track_number = atoi (parts[0]);
-					if (parts[1]) {
-						tag->track_count = atoi (parts[1]);
-					}
-				}
-				g_strfreev (parts);
-				g_free (word);
-
-				break;
-			}
-			case ID3V24_TYER:
-				if (atoi (word) > 0) {
-					tag->recording_time = tracker_date_guess (word);
-				}
-				g_free (word);
-				break;
-			default:
-				g_free (word);
+		case ID3V24_TYER:
+			if (atoi (word) > 0) {
+				tag->recording_time = tracker_date_guess (word);
 			}
+			g_free (word);
+			break;
+		default:
+			g_free (word);
 		}
-		}
-
-		pos += csize;
+	}
 	}
 }
 
 static void
-get_id3v20_tags (const gchar          *data,
-                 size_t                        size,
+get_id3v20_tags (id3v2frame            frame,
+                 const gchar          *data,
+                 size_t                csize,
                  id3tag               *info,
                  const gchar          *uri,
                  TrackerSparqlBuilder *metadata,
@@ -1523,125 +1477,97 @@ get_id3v20_tags (const gchar          *data,
 	id3v2tag *tag = &filedata->id3v22;
 	guint pos = 0;
 
-	while (pos < size) {
-		id3v2frame frame;
-		size_t csize;
+	if (frame == ID3V2_PIC) {
+		/* embedded image */
+		gchar          pic_type;
+		const gchar   *desc;
+		guint          offset;
+		const gchar   *mime;
 
-		if (pos + 6 > size)  {
-			return;
+		mime      = &data[pos + 3 + 1];
+		pic_type  =  data[pos + 3 + 1 + 3];
+		desc      = &data[pos + 3 + 1 + 3 + 1];
+
+		if (pic_type == 3 || (pic_type == 0 && filedata->albumart_size == 0)) {
+			offset = pos + 3 + 1 + 3 + 1 + strlen (desc) + 1;
+
+			filedata->albumart_mime = g_strdup (mime);
+			filedata->albumart_data = g_malloc0 (csize);
+			memcpy (filedata->albumart_data, &data[offset], csize);
+			filedata->albumart_size = csize;
+		}
+	} else {
+		/* text frames */
+		gchar *word;
+
+		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+		if (!tracker_is_empty_string (word)) {
+			g_strstrip (word);
 		}
 
-		frame = id3v2_get_frame (&data[pos]);
+		switch (frame) {
+		case ID3V2_COM:
+			tag->comment = word;
+			break;
+		case ID3V2_TAL:
+			tag->album = word;
+			break;
+		case ID3V2_TCO: {
+			gint genre;
 
-		csize = (((unsigned char)(data[pos + 3]) << 16) +
-		         ((unsigned char)(data[pos + 4]) << 8) +
-		         ((unsigned char)(data[pos + 5]) ) );
+			if (get_genre_number (word, &genre)) {
+				g_free (word);
+				word = g_strdup (get_genre_name (genre));
+			}
 
-		pos += 6;
+			if (word && strcasecmp (word, "unknown") != 0) {
+				tag->content_type = word;
+			} else {
+				g_free (word);
+			}
 
-		if (frame == ID3V2_UNKNOWN) {
-			/* ignore unknown frames */
-			pos += csize;
-			continue;
-		}
-
-		if (pos + csize > size) {
 			break;
-		} else if (csize == 0) {
-			continue;
 		}
-
-		if (frame == ID3V2_PIC) {
-			/* embedded image */
-			gchar          pic_type;
-			const gchar   *desc;
-			guint          offset;
-			const gchar   *mime;
-
-			mime      = &data[pos + 3 + 1];
-			pic_type  =  data[pos + 3 + 1 + 3];
-			desc      = &data[pos + 3 + 1 + 3 + 1];
-
-			if (pic_type == 3 || (pic_type == 0 && filedata->albumart_size == 0)) {
-				offset = pos + 3 + 1 + 3 + 1 + strlen (desc) + 1;
-
-				filedata->albumart_mime = g_strdup (mime);
-				filedata->albumart_data = g_malloc0 (csize);
-				memcpy (filedata->albumart_data, &data[offset], csize);
-				filedata->albumart_size = csize;
-			}
-		} else {
-			/* text frames */
-			gchar *word;
-
-			word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
-			if (!tracker_is_empty_string (word)) {
-				g_strstrip (word);
-			}
-
-			switch (frame) {
-			case ID3V2_COM:
-				tag->comment = word;
-				break;
-			case ID3V2_TAL:
-				tag->album = word;
-				break;
-			case ID3V2_TCO: {
-				gint genre;
-
-				if (get_genre_number (word, &genre)) {
-					g_free (word);
-					word = g_strdup (get_genre_name (genre));
-				}
-
-				if (word && strcasecmp (word, "unknown") != 0) {
-					tag->content_type = word;
-				} else {
-					g_free (word);
-				}
-
-				break;
-			}
-			case ID3V2_TCR:
-				tag->copyright = word;
-				break;
-			case ID3V2_TLE:
-				tag->length = atoi (word) / 1000;
-				g_free (word);
-				break;
-			case ID3V2_TPB:
-				tag->publisher = word;
-				break;
-			case ID3V2_TP1:
-				tag->performer1 = word;
-				break;
-			case ID3V2_TP2:
-				tag->performer2 = word;
-				break;
-			case ID3V2_TT1:
-				tag->title1 = word;
-				break;
-			case ID3V2_TT2:
-				tag->title2 = word;
-				break;
-			case ID3V2_TT3:
-				tag->title3 = word;
-				break;
-			case ID3V2_TXT:
-				tag->text = word;
-				break;
-			case ID3V2_TYE:
-				if (atoi (word) > 0) {
-					tag->recording_time = tracker_date_guess (word);
-				}
-				g_free (word);
-				break;
-			default:
-				g_free (word);
+		case ID3V2_TCR:
+			tag->copyright = word;
+			break;
+		case ID3V2_TEN:
+			tag->encoded_by = word;
+			break;
+		case ID3V2_TLE:
+			tag->length = atoi (word) / 1000;
+			g_free (word);
+			break;
+		case ID3V2_TPB:
+			tag->publisher = word;
+			break;
+		case ID3V2_TP1:
+			tag->performer1 = word;
+			break;
+		case ID3V2_TP2:
+			tag->performer2 = word;
+			break;
+		case ID3V2_TT1:
+			tag->title1 = word;
+			break;
+		case ID3V2_TT2:
+			tag->title2 = word;
+			break;
+		case ID3V2_TT3:
+			tag->title3 = word;
+			break;
+		case ID3V2_TXT:
+			tag->text = word;
+			break;
+		case ID3V2_TYE:
+			if (atoi (word) > 0) {
+				tag->recording_time = tracker_date_guess (word);
 			}
+			g_free (word);
+			break;
+		default:
+			g_free (word);
 		}
-
-		pos += csize;
 	}
 }
 
@@ -1701,15 +1627,63 @@ parse_id3v24 (const gchar           *data,
 		}
 	}
 
-	if (unsync) {
-		size_t unsync_size;
-		gchar *body;
+	while (pos < size) {
+		id3v24frame frame;
+		size_t csize;
+		unsigned short flags;
 
-		un_unsync (&data[pos], tsize, (unsigned char **) &body, &unsync_size);
-		get_id3v24_tags (body, unsync_size, info, uri, metadata, filedata);
-		g_free (body);
-	} else {
-		get_id3v24_tags (&data[pos], tsize, info, uri, metadata, filedata);
+		if (pos + 10 > size) {
+			return;
+		}
+
+		frame = id3v24_get_frame (&data[pos]);
+
+		csize = (((data[pos+4] & 0x7F) << 21) |
+		         ((data[pos+5] & 0x7F) << 14) |
+		         ((data[pos+6] & 0x7F) << 7) |
+		         ((data[pos+7] & 0x7F) << 0));
+
+		flags = (((unsigned char) (data[pos + 8]) << 8) +
+		         ((unsigned char) (data[pos + 9])));
+
+		pos += 10;
+
+		if (frame == ID3V24_UNKNOWN) {
+			/* ignore unknown frames */
+			pos += csize;
+			continue;
+		}
+
+		if (pos + csize > size) {
+			break;
+		} else if (csize == 0) {
+			continue;
+		}
+
+		if (((flags & 0x80) > 0) ||
+		    ((flags & 0x40) > 0)) {
+			pos += csize;
+			continue;
+		}
+
+		if ((flags & 0x20) > 0) {
+			/* The "group" identifier, skip a byte */
+			pos++;
+			csize--;
+		}
+
+		if ((flags & 0x02) || unsync) {
+			size_t unsync_size;
+			gchar *body;
+
+			un_unsync (&data[pos], csize, (unsigned char **) &body, &unsync_size);
+			get_id3v24_tags (frame, body, unsync_size, info, uri, metadata, filedata);
+			g_free (body);
+		} else {
+			get_id3v24_tags (frame, &data[pos], csize, info, uri, metadata, filedata);
+		}
+
+		pos += csize;
 	}
 
 	*offset_delta = tsize + 10;
@@ -1781,15 +1755,62 @@ parse_id3v23 (const gchar          *data,
 		}
 	}
 
-	if (unsync) {
-		size_t unsync_size;
-		gchar *body;
+	while (pos < size) {
+		id3v24frame frame;
+		size_t csize;
+		unsigned short flags;
 
-		un_unsync (&data[pos], tsize, (unsigned char **) &body, &unsync_size);
-		get_id3v23_tags (body, unsync_size, info, uri, metadata, filedata);
-		g_free (body);
-	} else {
-		get_id3v23_tags (&data[pos], tsize, info, uri, metadata, filedata);
+		if (pos + 10 > size) {
+			return;
+		}
+
+		frame = id3v24_get_frame (&data[pos]);
+
+		csize = (((unsigned char)(data[pos + 4]) << 24) |
+		         ((unsigned char)(data[pos + 5]) << 16) |
+		         ((unsigned char)(data[pos + 6]) << 8)  |
+		         ((unsigned char)(data[pos + 7]) << 0) );
+
+		flags = (((unsigned char)(data[pos + 8]) << 8) +
+		         ((unsigned char)(data[pos + 9])));
+
+		pos += 10;
+
+		if (frame == ID3V24_UNKNOWN) {
+			/* ignore unknown frames */
+			pos += csize;
+			continue;
+		}
+
+		if (pos + csize > size) {
+			break;
+		} else if (csize == 0) {
+			continue;
+		}
+
+		if (((flags & 0x80) > 0) || ((flags & 0x40) > 0)) {
+			pos += csize;
+			continue;
+		}
+
+		if ((flags & 0x20) > 0) {
+			/* The "group" identifier, skip a byte */
+			pos++;
+			csize--;
+		}
+
+		if ((flags & 0x02) || unsync) {
+			size_t unsync_size;
+			gchar *body;
+
+			un_unsync (&data[pos], csize, (unsigned char **) &body, &unsync_size);
+			get_id3v23_tags (frame, body, unsync_size, info, uri, metadata, filedata);
+			g_free (body);
+		} else {
+			get_id3v23_tags (frame, &data[pos], csize, info, uri, metadata, filedata);
+		}
+
+		pos += csize;
 	}
 
 	*offset_delta = tsize + 10;
@@ -1828,15 +1849,47 @@ parse_id3v20 (const gchar          *data,
 	}
 	pos = 10;
 
-	if (unsync) {
-		size_t  unsync_size;
-		gchar  *body;
+	while (pos < size) {
+		id3v2frame frame;
+		size_t csize;
 
-		un_unsync (&data[pos], tsize, (unsigned char **) &body, &unsync_size);
-		get_id3v20_tags (body, unsync_size, info, uri, metadata, filedata);
-		g_free (body);
-	} else {
-		get_id3v20_tags (&data[pos], tsize, info, uri, metadata, filedata);
+		if (pos + 6 > size)  {
+			return;
+		}
+
+		frame = id3v2_get_frame (&data[pos]);
+
+		csize = (((unsigned char)(data[pos + 3]) << 16) +
+		         ((unsigned char)(data[pos + 4]) << 8) +
+		         ((unsigned char)(data[pos + 5]) ) );
+
+		pos += 6;
+
+		if (frame == ID3V2_UNKNOWN) {
+			/* ignore unknown frames */
+			pos += csize;
+			continue;
+		}
+
+		if (pos + csize > size) {
+			break;
+		} else if (csize == 0) {
+			continue;
+		}
+
+		/* Early versions do not have unsynch per frame */
+		if (unsync) {
+			size_t  unsync_size;
+			gchar  *body;
+
+			un_unsync (&data[pos], csize, (unsigned char **) &body, &unsync_size);
+			get_id3v20_tags (frame, body, unsync_size, info, uri, metadata, filedata);
+			g_free (body);
+		} else {
+			get_id3v20_tags (frame, &data[pos], csize, info, uri, metadata, filedata);
+		}
+
+		pos += csize;
 	}
 
 	*offset_delta = tsize + 10;
@@ -1883,9 +1936,7 @@ extract_mp3 (const gchar          *uri,
 	goffset size;
 	goffset  buffer_size;
 	goffset audio_offset;
-	MP3Data md;
-
-	memset (&md, 0, sizeof (MP3Data));
+	MP3Data md = { 0 };
 
 	filename = g_filename_from_uri (uri, NULL, NULL);
 
@@ -1947,37 +1998,68 @@ extract_mp3 (const gchar          *uri,
 	/* Get other embedded tags */
 	audio_offset = parse_id3v2 (buffer, buffer_size, &md.id3v1, uri, metadata, &md);
 
-	md.title = tracker_coalesce (4, md.id3v24.title2, md.id3v23.title2, md.id3v22.title2, md.id3v1.title);
-	md.lyricist = tracker_coalesce (4, md.id3v24.text, md.id3v23.toly, md.id3v23.text, md.id3v22.text);
+	md.title = tracker_coalesce_strip (4, md.id3v24.title2,
+	                                   md.id3v23.title2,
+	                                   md.id3v22.title2,
+	                                   md.id3v1.title);
 
-	md.composer = tracker_coalesce (3,
-	                                md.id3v24.composer,
-	                                md.id3v23.composer,
-	                                md.id3v22.composer);
+	md.lyricist = tracker_coalesce_strip (4, md.id3v24.text,
+	                                      md.id3v23.toly,
+	                                      md.id3v23.text,
+	                                      md.id3v22.text);
 
-	md.performer = tracker_coalesce (7,
-	                                 md.id3v24.performer1, md.id3v24.performer2,
-	                                 md.id3v23.performer1, md.id3v23.performer2,
-	                                 md.id3v22.performer1, md.id3v22.performer2,
-	                                 md.id3v1.artist);
-	md.album = tracker_coalesce (4, md.id3v24.album, md.id3v23.album, md.id3v22.album, md.id3v1.album);
-	md.genre = tracker_coalesce (7,
-	                             md.id3v24.content_type, md.id3v24.title1,
-	                             md.id3v23.content_type, md.id3v23.title1,
-	                             md.id3v22.content_type, md.id3v22.title1,
-	                             md.id3v1.genre);
-	md.recording_time = tracker_coalesce (7,
-	                                      md.id3v24.recording_time, md.id3v24.release_time,
-	                                      md.id3v23.recording_time, md.id3v23.release_time,
-	                                      md.id3v22.recording_time, md.id3v22.release_time,
-	                                      md.id3v1.recording_time);
-	md.publisher = tracker_coalesce (3, md.id3v24.publisher, md.id3v23.publisher, md.id3v22.publisher);
-	md.copyright = tracker_coalesce (3, md.id3v24.copyright, md.id3v23.copyright, md.id3v22.copyright);
-	md.comment = tracker_coalesce (7,
-	                               md.id3v24.title3, md.id3v24.comment,
-	                               md.id3v23.title3, md.id3v23.comment,
-	                               md.id3v22.title3, md.id3v22.comment,
-	                               md.id3v1.comment);
+	md.composer = tracker_coalesce_strip (3, md.id3v24.composer,
+	                                      md.id3v23.composer,
+	                                      md.id3v22.composer);
+
+	md.performer = tracker_coalesce_strip (7, md.id3v24.performer1,
+	                                       md.id3v24.performer2,
+	                                       md.id3v23.performer1,
+	                                       md.id3v23.performer2,
+	                                       md.id3v22.performer1,
+	                                       md.id3v22.performer2,
+	                                       md.id3v1.artist);
+
+	md.album = tracker_coalesce_strip (4, md.id3v24.album,
+	                                   md.id3v23.album,
+	                                   md.id3v22.album,
+	                                   md.id3v1.album);
+
+	md.genre = tracker_coalesce_strip (7, md.id3v24.content_type,
+	                                   md.id3v24.title1,
+	                                   md.id3v23.content_type,
+	                                   md.id3v23.title1,
+	                                   md.id3v22.content_type,
+	                                   md.id3v22.title1,
+	                                   md.id3v1.genre);
+
+	md.recording_time = tracker_coalesce_strip (7, md.id3v24.recording_time,
+	                                            md.id3v24.release_time,
+	                                            md.id3v23.recording_time,
+	                                            md.id3v23.release_time,
+	                                            md.id3v22.recording_time,
+	                                            md.id3v22.release_time,
+	                                            md.id3v1.recording_time);
+
+	md.publisher = tracker_coalesce_strip (3, md.id3v24.publisher,
+	                                       md.id3v23.publisher,
+	                                       md.id3v22.publisher);
+
+	md.copyright = tracker_coalesce_strip (3, md.id3v24.copyright,
+	                                       md.id3v23.copyright,
+	                                       md.id3v22.copyright);
+
+	md.comment = tracker_coalesce_strip (7, md.id3v24.title3,
+	                                     md.id3v24.comment,
+	                                     md.id3v23.title3,
+	                                     md.id3v23.comment,
+	                                     md.id3v22.title3,
+	                                     md.id3v22.comment,
+	                                     md.id3v1.comment);
+
+	md.encoded_by = tracker_coalesce_strip (3, md.id3v24.encoded_by,
+						md.id3v23.encoded_by,
+						md.id3v22.encoded_by);
 
 	if (md.id3v24.track_number != 0) {
 		md.track_number = md.id3v24.track_number;
@@ -1998,7 +2080,7 @@ extract_mp3 (const gchar          *uri,
 	}
 
 	if (md.performer) {
-		md.performer_uri = tracker_uri_printf_escaped ("urn:artist:%s", md.performer);
+		md.performer_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", md.performer);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 
@@ -2009,11 +2091,10 @@ extract_mp3 (const gchar          *uri,
 		tracker_sparql_builder_object_unvalidated (preupdate, md.performer);
 
 		tracker_sparql_builder_insert_close (preupdate);
-		/* do not delete artist, needed by albumart */
 	}
 
 	if (md.composer) {
-		md.composer_uri = tracker_uri_printf_escaped ("urn:artist:%s", md.composer);
+		md.composer_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", md.composer);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 
@@ -2027,7 +2108,7 @@ extract_mp3 (const gchar          *uri,
 	}
 
 	if (md.lyricist) {
-		md.lyricist_uri = tracker_uri_printf_escaped ("urn:artist:%s", md.lyricist);
+		md.lyricist_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", md.lyricist);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, md.lyricist_uri);
@@ -2036,11 +2117,10 @@ extract_mp3 (const gchar          *uri,
 		tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.lyricist);
 		tracker_sparql_builder_insert_close (preupdate);
-		g_free (md.lyricist);
 	}
 
 	if (md.album) {
-		md.album_uri = tracker_uri_printf_escaped ("urn:album:%s", md.album);
+		md.album_uri = tracker_sparql_escape_uri_printf ("urn:album:%s", md.album);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 
@@ -2081,7 +2161,6 @@ extract_mp3 (const gchar          *uri,
 	if (md.title) {
 		tracker_sparql_builder_predicate (metadata, "nie:title");
 		tracker_sparql_builder_object_unvalidated (metadata, md.title);
-		/* do not delete title, needed by albumart */
 	}
 
 
@@ -2112,25 +2191,21 @@ extract_mp3 (const gchar          *uri,
 	if (md.recording_time) {
 		tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
 		tracker_sparql_builder_object_unvalidated (metadata, md.recording_time);
-		g_free (md.recording_time);
 	}
 
 	if (md.genre) {
 		tracker_sparql_builder_predicate (metadata, "nfo:genre");
 		tracker_sparql_builder_object_unvalidated (metadata, md.genre);
-		g_free (md.genre);
 	}
 
 	if (md.copyright) {
 		tracker_sparql_builder_predicate (metadata, "nie:copyright");
 		tracker_sparql_builder_object_unvalidated (metadata, md.copyright);
-		g_free (md.copyright);
 	}
 
 	if (md.comment) {
 		tracker_sparql_builder_predicate (metadata, "nie:comment");
 		tracker_sparql_builder_object_unvalidated (metadata, md.comment);
-		g_free (md.comment);
 	}
 
 	if (md.publisher) {
@@ -2141,7 +2216,11 @@ extract_mp3 (const gchar          *uri,
 		tracker_sparql_builder_predicate (metadata, "nco:fullname");
 		tracker_sparql_builder_object_unvalidated (metadata, md.publisher);
 		tracker_sparql_builder_object_blank_close (metadata);
-		g_free (md.publisher);
+	}
+
+	if (md.encoded_by) {
+		tracker_sparql_builder_predicate (metadata, "nfo:encodedBy");
+		tracker_sparql_builder_object_unvalidated (metadata, md.encoded_by);
 	}
 
 	if (md.track_number > 0) {
@@ -2149,6 +2228,10 @@ extract_mp3 (const gchar          *uri,
 		tracker_sparql_builder_object_int64 (metadata, md.track_number);
 	}
 
+	/* FIXME We use a hardcoded value here for now. In reality there's a second option MP3X */
+	tracker_sparql_builder_predicate (metadata, "nmm:dlnaProfile");
+	tracker_sparql_builder_object_string (metadata, "MP3");
+
 	/* Get mp3 stream info */
 	mp3_parse (buffer, buffer_size, audio_offset, uri, metadata, &md);
 
@@ -2159,14 +2242,10 @@ extract_mp3 (const gchar          *uri,
 	                          md.album,
 	                          filename);
 
-	g_free (md.performer);
-	g_free (md.composer);
-	g_free (md.album);
-	g_free (md.title);
-	g_free (md.albumart_data);
-	g_free (md.albumart_mime);
-
-	g_free (md.id3v1.encoding);
+	id3v2tag_free (&md.id3v22);
+	id3v2tag_free (&md.id3v23);
+	id3v2tag_free (&md.id3v24);
+	id3tag_free (&md.id3v1);
 
 #ifndef G_OS_WIN32
 	munmap (buffer, buffer_size);
