@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -25,12 +25,17 @@
 
 #include <dbus/dbus-glib-bindings.h>
 
+#include "tracker-status.h"
+
 G_BEGIN_DECLS
 
-gboolean    tracker_dbus_init                    (void);
-void        tracker_dbus_shutdown                (void);
-gboolean    tracker_dbus_register_objects        (void);
-GObject    *tracker_dbus_get_object              (GType type);
+gboolean        tracker_dbus_init                          (void);
+void            tracker_dbus_shutdown                      (void);
+gboolean        tracker_dbus_register_objects              (void);
+GObject        *tracker_dbus_get_object                    (GType type);
+TrackerStatus  *tracker_dbus_register_notifier             (void);
+gboolean        tracker_dbus_register_names                (void);
+gboolean        tracker_dbus_register_class_signal_objects (void);
 
 G_END_DECLS
 
