@@ -26,10 +26,13 @@
 
 #include "tracker-events.h"
 
+#define MAX_EVENTS 1000000
 
 typedef struct {
 	GHashTable *allowances;
 	GArray *events;
+	GStringChunk *chunk;
+	gboolean frozen;
 } EventsPrivate;
 
 static EventsPrivate *private;
@@ -70,10 +73,14 @@ prepare_event_for_rdf_type (EventsPrivate *private,
 		private->events = g_array_new (TRUE, FALSE, sizeof (TrackerEvent));
 	}
 
+	if (!private->chunk) {
+		private->chunk = g_string_chunk_new (4096);
+	}
+
 	event.type = type;
 	event.class = rdf_class;
 	event.predicate = tracker_ontologies_get_property_by_uri (predicate);
-	event.subject = g_strdup (uri);
+	event.subject = g_string_chunk_insert_const (private->chunk, uri);
 
 	g_array_append_val (private->events, event);
 }
@@ -88,6 +95,18 @@ tracker_events_insert (const gchar *uri,
 	g_return_if_fail (rdf_types || type != TRACKER_DBUS_EVENTS_TYPE_UPDATE);
 	g_return_if_fail (private != NULL);
 
+	if (private->frozen) {
+		return;
+	}
+
+	if (private->events && private->events->len > MAX_EVENTS) {
+		/* too many events would lead to high memory usage and to dbus signal messages
+		   larger than maximum dbus message size, which results in dbus disconnect and tracker-store crash */
+		g_warning ("Too many events pending, disabling events for current transaction");
+		tracker_events_reset ();
+		private->frozen = TRUE;
+	}
+
 	if (rdf_types && type == TRACKER_DBUS_EVENTS_TYPE_UPDATE) {
 		guint i;
 
@@ -111,27 +130,26 @@ tracker_events_insert (const gchar *uri,
 	}
 }
 
-static void
-tracker_event_destroy (TrackerEvent *event)
-{
-	g_free (event->subject);
-}
-
 void
 tracker_events_reset (void)
 {
-	guint i;
-
 	g_return_if_fail (private != NULL);
 
 	if (private->events) {
-		for (i = 0; i < private->events->len; i++) {
-			tracker_event_destroy (&g_array_index (private->events, TrackerEvent, i));
-		}
 		g_array_free (private->events, TRUE);
-
+		g_string_chunk_free (private->chunk);
+		private->chunk = NULL;
 		private->events = NULL;
 	}
+	private->frozen = FALSE;
+}
+
+void
+tracker_events_freeze (void)
+{
+	g_return_if_fail (private != NULL);
+
+	private->frozen = TRUE;
 }
 
 GArray *
